(function() {
    'use strict';

    angular.module('asdApp')
        .controller('AccountSalesDashboardCtrl', ['$scope', '$q', 'currentAccount', 'appConstant', 'userService',
            'dataMapTemplateService', 'analyticsDataService', 'accountSalesSummaryService', 'activityDataService',
            'chartDataManager', 'messageService', 'chartJsPlugins', 'Utilities', AccountSalesDashboardCtrl]);

    function AccountSalesDashboardCtrl($scope, $q, currentAccount, appConstant, userService, dataMapTemplateService,
                                       analyticsDataService, accountSalesSummaryService, activityDataService, chartDataManager,
                                       messageService, chartJsPlugins, Utilities) {
        var dash = this;

        var MONTHLY_TEXT = 'monthly';

        var WEEKLY_TEXT = 'weekly';

        var YEAR_MONTH_DATE = 'YYYY-MM-DD';

        var Three_CHAR_MONTH_DATE_FORMAT = 'MMM D';

        var FULL_DATE_TIME_FORMAT = 'YYYY-MM-DDTHH:mm:ss.SSSZ';

        var LOWERCASE_W_TEXT = 'w';

        var UPPERCASE_M_TEXT = 'M';

        var GMT = "GMT";

        var utils = new window.VeevaUtilities();
        dash.account = currentAccount;

        // Flag to show/hide dashboard content
        dash.salesDataReady = false;
        dash.activityDataReady = true;
        dash.noSalesData = true;

        // Metadata filter options for Sales data
        dash.marketOptions = [];
        dash.dataTypeOptions = [];
        dash.scaleOptions = [];
        dash.periodOptions = [];

        // Product list
        dash.analyticsProductGroupSet = {};

        // Product filter options for the Chart
        dash.myProductOptions = [];
        dash.competitorProductOptions = [];

        // Helper objects for Sales data
        dash.salesSummaryByCompanyPgId = {};
        dash.salesSummaryByCompetitorPgId = {};
        dash.companyProductToSalesSummaryMapByDate = {};
        dash.competitorProductToSalesSummaryMapByDate = {};

        // Helper objects for Activity data
        dash.callDetailByProductId = {};
        dash.callSampleByProductId = {};
        dash.sentEmailsByProductId = {};

        // Set up moment global locale
        moment.locale(userService.getUserLocale());
        // Get user timezone
        var user_timezone = userService.getUserTimezone();
        if(!moment.tz.zone(user_timezone)) {
            user_timezone = GMT;
        }

        function isMatchingCurrentScale(selectedScaleName, scaleOfDataType) {
            return selectedScaleName === scaleOfDataType;
        }

        function isFoundInMarketList(marketList, selectedMarketId) {
            return marketList.indexOf(selectedMarketId) >= 0;
        }

        function isEligibleForDisplay(marketList, selectedMarketId, selectedScaleName, scaleOfDataType, marketIdOfDataType) {
            if (isMatchingCurrentScale(selectedScaleName, scaleOfDataType)) {
                if (isFoundInMarketList(marketList, selectedMarketId) && selectedMarketId === marketIdOfDataType) {
                    return true;
                } else if (!isFoundInMarketList(marketList, selectedMarketId) && !marketIdOfDataType) {
                    return true;
                }
            }

            return false;
        }

        registerChartPlugin();

        $q.all(getSalesMetadata()).then(function() {
            initializeMetadataFilter();

            accountSalesSummaryService.getAccountSalesSummary(dash.selectedScale.apiName, dash.selectedDataType.name, dash.selectedMarket.id, currentAccount.Id).then(function(summary) {
                if(summary.length) {
                    dash.noSalesData = false;
                }
                dash.salesDataLastUpdatedDate = accountSalesSummaryService.getFormattedSalesDataDate();
                dash.timelineArray = generateTimelineArray(getStartDateForSelectedPeriod());

                getProductList().then(function() {
                    salesSummaryDataProcessor(summary);
                    salesChartDataProcessor();
                    profileChartDataProcessor();
                    salesTableDataProcessor();

                    getActivityData().then(function() {
                        activityChartDataProcessor();

                    }, function() {
                        console.error(appConstant.ERROR_ACTIVITY_DATA);

                    }).finally(function() {
                        initializeActivityChart();
                        updateActivityTableDataModel();
                        $scope.$broadcast('updateBarChart');
                    });

                    chartAndTableDataModelBulkUpdate();
                    dash.salesDataReady = true;
                });

            }, function() {
                dash.noSalesData = true;
                salesDataCleanUp();
                console.error(appConstant.ERROR_SALES_DATA);

                chartAndTableDataModelBulkUpdate();
                dash.salesDataReady = true;
            });
        });

        /**
         * DOM event listeners
         */
        dash.metadataFilterOnChange = function() {
            var market = dash.selectedMarket.id;
            var dataType = dash.selectedDataType.name;
            var scale = dash.selectedScale.apiName;

            console.log('scale=' + scale + ', dataType=' + dataType + ', market=' + market + ', accountId=' + currentAccount.Id);
            accountSalesSummaryService.getAccountSalesSummary(scale, dataType, market, currentAccount.Id).then(function(summary) {
                dash.noSalesData = !summary.length;
                dash.salesDataLastUpdatedDate = accountSalesSummaryService.getFormattedSalesDataDate();
                dash.timelineArray = generateTimelineArray(getStartDateForSelectedPeriod());

                getProductList().then(function() {
                    salesSummaryDataProcessor(summary);
                    salesChartDataProcessor();
                    profileChartDataProcessor();
                    salesTableDataProcessor();

                    getActivityData().then(function() {
                        activityChartDataProcessor();

                    }, function() {
                        activityDataCleanUp();
                        console.error(appConstant.ERROR_ACTIVITY_DATA);

                    }).finally(function() {
                        initializeActivityChart();
                        updateActivityTableDataModel();
                        $scope.$broadcast('updateBarChart');
                    });

                    chartAndTableDataModelBulkUpdate();
                    $scope.$broadcast('updateProfileChart');
                    $scope.$broadcast('updateMainLineChart');
                    $scope.$broadcast('updateBarChart');
                });

            }, function() {
                dash.noSalesData = true;
                chartDataManager.clearChartData();
                salesDataCleanUp();
                console.error(appConstant.ERROR_SALES_DATA);

                chartAndTableDataModelBulkUpdate();
                $scope.$broadcast('updateProfileChart');
                $scope.$broadcast('updateMainLineChart');
                $scope.$broadcast('updateBarChart');
            });
        };

        dash.periodFilterOnChange = function() {
            dash.timelineArray = generateTimelineArray(getStartDateForSelectedPeriod());

            salesChartDataProcessor();
            updateSalesChartDataModel();

            profileChartDataProcessor();
            updateProfileChartDataModel();

            salesTableDataProcessor();
            updateSalesTableDataModel();

            getActivityData().then(function() {
                activityChartDataProcessor();
                dash.activityDataReady = true;

            }, function() {
                activityDataCleanUp();
                dash.activityDataReady = false;
                console.error(appConstant.ERROR_ACTIVITY_DATA);

            }).finally(function() {
                updateActivityChartDataModel();
                updateActivityTableDataModel();
                $scope.$broadcast('updateBarChart');
            });

            $scope.$broadcast('updateProfileChart');
            $scope.$broadcast('updateMainLineChart');
        };

        dash.myProductOnChange = function() {
            salesChartDataProcessor();
            updateSalesChartDataModel();
            $scope.$broadcast('updateMainLineChart');

            activityChartDataProcessor();
            updateActivityChartDataModel();
            $scope.$broadcast('updateBarChart');
        };

        dash.competitorProductOnChange = function() {
            salesChartDataProcessor();
            updateSalesChartDataModel();

            $scope.$broadcast('updateMainLineChart');
            $scope.$broadcast('updateBarChart');
        };

        dash.activityTypeOnChange = function() {
            updateActivityChartDataModel();
            $scope.$broadcast('updateBarChart');
        };

        dash.initializeDataTypeFilter = function(){
            var dataTypeOptions = [];
            var sortedOptions;

            // filtering dataType
            _.forEach(dash.analyticsFiles, function(row) {
                var marketId = row.Market_vod__c;
                var scaleVal = row.Scale_vod__c;
                var dataType1 = row.Column_Label_vod__c;
                var dataType2 = row.Second_Column_Label_vod__c;

                if (isEligibleForDisplay(dash.marketLevelOne, dash.selectedMarket.id, dash.selectedScale.apiName, scaleVal, marketId)) {
                    if(Utilities.isNotEmptyString(dataType1)) {
                        dataTypeOptions.push({
                            name: dataType1
                        });
                    }

                    if(Utilities.isNotEmptyString(dataType2)) {
                        dataTypeOptions.push({
                            name: dataType2
                        });
                    }
                }
            });

            if(dataTypeOptions.length === 0){
                dataTypeOptions.push({
                    name: appConstant.NO_DATA_AVAILABLE
                });
            }

            sortedOptions = _.sortBy(dataTypeOptions, function(option){
                return option.name;
            });

            dash.dataTypeOptions = _.uniq(sortedOptions, true, function(dataType){
                return dataType.name
            });

            dash.selectedDataType = dash.dataTypeOptions[0];
        };

        dash.initializePeriodFilter = function() {
            dash.periodOptions = [];

            if(dash.selectedScale.apiName === 'Month_vod') {
                dash.periodOptions.push({
                    name: messageService.getMessage('rollingXMonth', appConstant.MONTH_1.value),
                    meta: appConstant.MONTH_1
                });
                dash.periodOptions.push({
                    name: messageService.getMessage('rollingXMonth', appConstant.MONTH_3.value),
                    meta: appConstant.MONTH_3
                });
                dash.periodOptions.push({
                    name: messageService.getMessage('rollingXMonth', appConstant.MONTH_6.value),
                    meta: appConstant.MONTH_6
                });
                dash.periodOptions.push({
                    name: messageService.getMessage('rollingXMonth', appConstant.MONTH_12.value),
                    meta: appConstant.MONTH_12
                });
            } else if(dash.selectedScale.apiName === 'Week_vod') {
                dash.periodOptions.push({
                    name: messageService.getMessage('rollingXWeeks', appConstant.WEEK_4.value),
                    meta: appConstant.WEEK_4
                });
                dash.periodOptions.push({
                    name: messageService.getMessage('rollingXWeeks', appConstant.WEEK_13.value),
                    meta: appConstant.WEEK_13
                });
                dash.periodOptions.push({
                    name: messageService.getMessage('rollingXWeeks', appConstant.WEEK_24.value),
                    meta: appConstant.WEEK_24
                });
                dash.periodOptions.push({
                    name: messageService.getMessage('rollingXWeeks', appConstant.WEEK_52.value),
                    meta: appConstant.WEEK_52
                });
            }

            dash.selectedPeriod = dash.periodOptions[0];
        };

        function getSalesMetadata() {
            var dataMapTemplateDeferred = $q.defer();
            var analyticsMarketDeferred = $q.defer();

            accountSalesSummaryService.fetchAnalyticsFilesReport().then(function(analyticsFiles){
                var scaleRecord = {};
                var marketRecord = {};
                var scaleOptions = [];
                var marketOptions = [];
                dash.analyticsFiles = analyticsFiles;
                angular.forEach(analyticsFiles, function(row) {
                    var scaleName = row.Scale_vod__c;
                    var marketLevelOne = row.Market_vod__c;

                    if(Utilities.isNotEmptyString(marketLevelOne) && !angular.isDefined(marketRecord[marketLevelOne])) {
                        marketOptions.push(marketLevelOne);
                        marketRecord[marketLevelOne] = true;
                    }
                    if(Utilities.isNotEmptyString(scaleName) && !angular.isDefined(scaleRecord[scaleName])) {
                        scaleOptions.push({
                            name: scaleName
                        });
                        scaleRecord[scaleName] = true;
                    }
                });

                dash.scaleOptions = messageService.getPicklistLabels("Analytics_Files_vod__c", "Scale_vod__c");

                dash.marketLevelOne = _.sortBy(marketOptions, function(option) {
                    return Utilities.getFieldValue(option, 'name', 0);
                });

                dataMapTemplateDeferred.resolve();
            });

            analyticsDataService.getAnalyticsMarkets().then(function(resp) {
                var marketOptions = [];

                angular.forEach(resp, function(market) {
                    marketOptions.push({
                        name: market.Name,
                        id: market.Id
                    });
                });

                // Sort the options by name
                dash.marketOptions = _.sortBy(marketOptions, function(option) {
                    return option.name;
                });

                analyticsMarketDeferred.resolve();
            });

            return [dataMapTemplateDeferred.promise, analyticsMarketDeferred.promise];
        }

        function getActivityData() {
            var deferred = $q.defer();
            var callDetailDeferred = $q.defer();
            var callSampleDeferred = $q.defer();
            var sentEmailDeferred = $q.defer();
            var productDeferred = $q.defer();

            var dateArray = dash.timelineArray;
            var latestPeriodStartDate = null;
            var oldestPeriodStartDate = null;
            var startDate = null;
            var endDate = null;

            if(dateArray.length) {
                latestPeriodStartDate = _.last(dateArray).apiFormat;
                oldestPeriodStartDate = _.first(dateArray).apiFormat;
            }

            var rollbackFrequency = dash.selectedPeriod.meta.frequency;

            startDate = moment.tz(oldestPeriodStartDate, YEAR_MONTH_DATE, user_timezone).toISOString();

            if(rollbackFrequency === WEEKLY_TEXT) {
                endDate = moment.tz(latestPeriodStartDate, YEAR_MONTH_DATE, user_timezone).add(1, LOWERCASE_W_TEXT).toISOString();
            } else if(rollbackFrequency === MONTHLY_TEXT) {
                endDate = moment.tz(latestPeriodStartDate, YEAR_MONTH_DATE, user_timezone).add(1, UPPERCASE_M_TEXT).toISOString();
            }

            activityDataService.getCalls(currentAccount.Id, startDate, endDate).then(function() {

                activityDataService.getCallDetailByProductId(currentAccount.Id, startDate, endDate).then(function(resp) {
                    console.log(resp);
                    dash.callDetailByProductId = resp;
                    callDetailDeferred.resolve();
                });

                activityDataService.getCallSampleByProductId(currentAccount.Id, startDate, endDate).then(function(resp) {
                    console.log(resp);
                    dash.callSampleByProductId = resp;
                    callSampleDeferred.resolve();
                });

                activityDataService.getSentEmailActivityByProductId(currentAccount.Id, startDate, endDate).then(function(resp) {
                    console.log(resp);
                    dash.sentEmailsByProductId = resp;
                    sentEmailDeferred.resolve();
                });

                activityDataService.getProductParentMap().then(function(resp) {
                    dash.productParentMap = resp;
                    productDeferred.resolve();
                });
            });

            $q.all([callDetailDeferred.promise, callSampleDeferred.promise, sentEmailDeferred.promise, productDeferred.promise]).then(function() {
                deferred.resolve();
            });

            return deferred.promise;
        }

        function getProductList() {
            var deferred = $q.defer();

            analyticsDataService.getProductList(dash.selectedMarket.id).then(function(resp) {
                dash.analyticsProductGroupSet = _.indexBy(resp, function(apg) {
                    return apg.Id;
                });
                deferred.resolve();
            });

            return deferred.promise;
        }

        function initializeMetadataFilter() {
            dash.selectedMarket = dash.marketOptions[0];
            dash.selectedScale = dash.scaleOptions[0];

            // We will initialize period options here since it depends on the value from Scale
            dash.initializePeriodFilter();
            dash.initializeDataTypeFilter();
        }

        function initializeActivityChart() {
            dash.activityType = [{
                id: 'calls'
            }, {
                id: 'emails'
            }];

            dash.selectedActivityType = dash.activityType[0];

            updateActivityChartDataModel();
        }

        function initializeSalesTable() {
            dash.productSalesTable = {
                header: [{
                    name: 'product',
                    display: messageService.getMessage('product')
                }, {
                    name: 'rollingValue',
                    display: messageService.getMessage('rollPeriod', dash.selectedDataType.name)
                }, {
                    name: 'previousValue',
                    display: messageService.getMessage('previousPeriod', dash.selectedDataType.name)
                }],
                data: []
            };

            updateSalesTableDataModel();
        }

        function initializeActivityTable() {
            dash.productActivityTable = {
                header: [{
                    name: 'product',
                    display: messageService.getMessage('product')
                }, {
                    name: 'calls',
                    display: messageService.getMessage('callsCLMPercent')
                }, {
                    name: 'callsPerWeek',
                    display: messageService.getMessage('callPerWeek')
                }, {
                    name: 'sentEmails',
                    display: messageService.getMessage('sentEmailsOpenRate')
                }, {
                    name: 'sentEmailsPerWeek',
                    display: messageService.getMessage('sentEmailPerWeek')
                }, {
                    name: 'samples',
                    display: messageService.getMessage('samples')
                }],
                data: []
            };
        }

        /**
         * This function process the sales summary data in the following ways
         * 1. Re-format the sales summary data to a more usage friendly structure as below:
         *      - Group the summary by company and competitor product group id
         *      - Index the company and competitor summary data first by date, then by product group id
         * 2. Capture and store available product options for company and competitor.
         * 3. Initialize product selection model for the Chart
         *
         * @param summary - Sales summary data
         */
        function salesSummaryDataProcessor(summary) {
            // Clean up
            dash.salesSummaryByCompanyPgId = {};
            dash.salesSummaryByCompetitorPgId = {};
            dash.myProductOptions = [];
            dash.competitorProductOptions = [];
            dash.companyProductToSalesSummaryMapByDate = {};
            dash.competitorProductToSalesSummaryMapByDate = {};

            // Sales summary data group by product group id
            var summaryByPgId = _.groupBy(summary, function(summaryRow) {
                return summaryRow.Product_Group_Id_vod__c;
            });

            var tempMyProductOptions = [];
            var tempCompetitorProductOptions = [];

            angular.forEach(dash.analyticsProductGroupSet, function(apg, apgId) {
                var summary = summaryByPgId[apgId] || [];

                if(analyticsDataService.isCompanyAnalyticsProductGroup(apg)) {
                    dash.salesSummaryByCompanyPgId[apgId] = summary;
                    tempMyProductOptions.push({
                        productGroupId: apgId,
                        productGroupName: apg.Display_Name_vod__c
                    });

                    // Fill out company product id to sales summary(value) map by date(key)
                    dash.companyProductToSalesSummaryMapByDate[apgId] = analyticsDataService.indexSalesSummaryByDate(dash.salesSummaryByCompanyPgId[apgId]);

                } else {
                    dash.salesSummaryByCompetitorPgId[apgId] = summary;
                    tempCompetitorProductOptions.push({
                        productGroupId: apgId,
                        productGroupName: apg.Display_Name_vod__c
                    });

                    // Fill out competitor product id to sales summary(value) map by date(key)
                    dash.competitorProductToSalesSummaryMapByDate[apgId] = analyticsDataService.indexSalesSummaryByDate(dash.salesSummaryByCompetitorPgId[apgId]);
                }
            });

            dash.myProductOptions = _.sortBy(tempMyProductOptions, function(option) {
                return option.productGroupName;
            });

            dash.competitorProductOptions = _.sortBy(tempCompetitorProductOptions, function(option) {
                return option.productGroupName;
            });

            // Add default option("None") for competitor product
            dash.competitorProductOptions.unshift({
                productGroupId: null,
                productGroupName: 'None'
            });

            dash.selectedMyProduct = dash.myProductOptions[0];
            dash.selectedCompetitorProduct = dash.competitorProductOptions[0];

            console.log(dash.salesSummaryByCompanyPgId);
            console.log(dash.salesSummaryByCompetitorPgId);
            console.log(dash.companyProductToSalesSummaryMapByDate);
        }

        function getStartDateForSelectedPeriod(){
            var dataDate = accountSalesSummaryService.getSalesDataDate();
            var dataDateMoment = moment.utc(dataDate, YEAR_MONTH_DATE);
            var rollbackFrequency = dash.selectedPeriod.meta.frequency;

            if(rollbackFrequency === MONTHLY_TEXT) {
                dataDateMoment.date(1);
            } else if(rollbackFrequency === WEEKLY_TEXT) {
                // set the day of the week, with Sunday as 0 and Saturday as 6
                // here we set the date to the past Monday
                if(dataDateMoment.day() === 0) {
                    dataDateMoment.day(-6);
                } else {
                    dataDateMoment.day(1);
                }
            }

            return dataDateMoment.format(YEAR_MONTH_DATE);
        }

        /**
         * Generate a map containing the aggregated sales value for each product group, which has sales data, and its parent.
         * @param dArray (optional) limiting the date range for which the data aggregation should happen. If omit, use
         * the current timelineArray
         * @returns {{Object}} a map containing the product aggregated value, first group by product group id, then group by date
         */
        function getProductAggregatedValueByDate(dArray) {
            var dateArray = angular.isArray(dArray) ?  dArray : dash.timelineArray;
            var productSalesSummaryByDate = [];
            var childProdArr = [];
            var aggregatedValByProduct = {};
            var tempArr = [];
            var aggregatedValByProductByDate = {};

            // Process data aggregation one day at a time
            angular.forEach(dateArray, function(date) {
                // Initiate the childProdArr to contain every product group with available sales data
                angular.forEach(dash.salesSummaryByCompanyPgId, function(summary, apgId) {
                    if(summary.length > 0) {
                        childProdArr.push(apgId);
                    }
                });
                angular.forEach(dash.salesSummaryByCompetitorPgId, function(summary, apgId) {
                    if(summary.length > 0) {
                        childProdArr.push(apgId);
                    }
                });

                // Loop through each product group to aggregate child data
                while (childProdArr.length > 0) {
                    childProdArr.forEach(function (pgId, index, object) {
                        object.splice(index, 1);

                        // Initialize product summary by date map for the current product group
                        if (analyticsDataService.isCompanyAnalyticsProductGroup(dash.analyticsProductGroupSet[pgId])) {
                            productSalesSummaryByDate = analyticsDataService.indexSalesSummaryByDate(dash.salesSummaryByCompanyPgId[pgId]);
                        } else {
                            productSalesSummaryByDate = analyticsDataService.indexSalesSummaryByDate(dash.salesSummaryByCompetitorPgId[pgId]);
                        }

                        // If there is direct parent, roll up data to this parent
                        var parentId = Utilities.getFieldValue(dash.analyticsProductGroupSet[pgId], 'Parent_Product_Group_vod__c', "");

                        if (Utilities.isNotEmptyString(parentId)) {
                            // initialize data for this product if no one has claimed it as parent yet
                            if (!aggregatedValByProduct.hasOwnProperty(pgId)) {
                                if (productSalesSummaryByDate[date.apiFormat] && productSalesSummaryByDate[date.apiFormat].Sales_Data_vod__c) {
                                    var productVal = parseFloat(productSalesSummaryByDate[date.apiFormat].Sales_Data_vod__c);

                                    if(aggregatedValByProduct.hasOwnProperty(pgId)) {
                                        aggregatedValByProduct[pgId] += productVal;
                                    } else {
                                        aggregatedValByProduct[pgId] = productVal;
                                    }

                                    // Has this parent registered yet? If not, create a record for it
                                    if (!aggregatedValByProduct.hasOwnProperty(parentId)) {
                                        aggregatedValByProduct[parentId] = 0;
                                    }
                                    // Roll up product data to its parent
                                    aggregatedValByProduct[parentId] += productVal;
                                }
                            } else {
                                aggregatedValByProduct[parentId] += aggregatedValByProduct[pgId];
                            }

                            if (tempArr.indexOf(parentId) === -1) {
                                tempArr.push(parentId);
                            }
                            if (object.length === 0) {
                                for (var i = 0; i < tempArr.length; i++) {
                                    object.push(tempArr[i]);
                                }
                                tempArr = [];
                            }

                        } else {
                            var myValue = 0;
                            var myChildValue = aggregatedValByProduct.hasOwnProperty(pgId) ? aggregatedValByProduct[pgId] : 0;

                            if(productSalesSummaryByDate[date.apiFormat] && productSalesSummaryByDate[date.apiFormat].Sales_Data_vod__c) {
                                myValue = parseFloat(productSalesSummaryByDate[date.apiFormat].Sales_Data_vod__c);
                            }
                            aggregatedValByProduct[pgId] = myValue + myChildValue;
                        }
                    });
                }

                aggregatedValByProductByDate[date.apiFormat] = utils.copyObject(aggregatedValByProduct);
                aggregatedValByProduct = {};
            });

            return aggregatedValByProductByDate;
        }

        /**
         * Filter out child product(leaf) and return them as map based on product group id
         */
        function findChildProducts(){
            var childProd = {};
            var parentId, gradParentId, parentArr = [], gradParentArr = [];
            angular.forEach(dash.analyticsProductGroupSet, function(pgId) {
                parentId = dash.analyticsProductGroupSet[pgId.Id].Parent_Product_Group_vod__c;
                if(parentId) {
                    parentArr.push(parentId);
                }
            });
            angular.forEach(dash.analyticsProductGroupSet, function(pgId) {
                if(parentArr.indexOf(pgId.Id) === -1) {
                    parentId = dash.analyticsProductGroupSet[pgId.Id].Parent_Product_Group_vod__c;
                    if(parentId) {
                        childProd[pgId.Id] = {parentId: parentId};
                        if (dash.analyticsProductGroupSet[parentId]) {
                            gradParentId = dash.analyticsProductGroupSet[parentId].Parent_Product_Group_vod__c;
                            if (gradParentId && gradParentArr.indexOf(gradParentId) === -1) {
                                gradParentArr.push(gradParentId);
                            }
                        }
                    }
                }
            });
            for (var key in childProd) {
                if (gradParentArr.indexOf(childProd[key].parentId) >= 0) {
                    delete childProd[key];
                }
            }
            return childProd;
        }

        /**
         * Generate sales chart labels and datasets
         */
        function salesChartDataProcessor() {
            // Form dateArray and dataArray, which will be used for chart's labels and dataset accordingly
            var labelArray = [];
            var mySalesDataArray = [];
            var competitorSalesDataArray = [];
            var productAggValByDate = getProductAggregatedValueByDate();

            dash.timelineArray.forEach(function(date) {
                if(dash.selectedMyProduct && dash.selectedMyProduct.productGroupId) {
                    var myProdAggVal = productAggValByDate[date.apiFormat][dash.selectedMyProduct.productGroupId];
                    mySalesDataArray.push(myProdAggVal || 0);
                }
                if(dash.selectedCompetitorProduct && dash.selectedCompetitorProduct.productGroupId) {
                    var compAggVal = productAggValByDate[date.apiFormat][dash.selectedCompetitorProduct.productGroupId];
                    competitorSalesDataArray.push(compAggVal || 0);
                }
                labelArray.push(date.displayFormat);
            });

            chartDataManager.setChartLabels(labelArray);
            chartDataManager.setMyProductSalesChartData(mySalesDataArray);
            chartDataManager.setCompetitorProductSalesChartData(competitorSalesDataArray);
        }

        /**
         * Generate profile chart datasets, including:
         * 1. Total market sales data
         * 2. Total sales data for company products
         * 3. Rate of change between current and previous period for market sales data
         * 4. Rate of change between current and previous period for company product sales data
         */
        function profileChartDataProcessor() {
            var allCompanySalesGroupByDate = analyticsDataService.groupSalesSummaryByDate(_.flatten(_.values(dash.salesSummaryByCompanyPgId)));
            var allCompetitorSalesGroupByDate = analyticsDataService.groupSalesSummaryByDate(_.flatten(_.values(dash.salesSummaryByCompetitorPgId)));

            var previousPeriodStartDate;

            dash.marketSalesSum = 0;
            dash.allCompanyProductSalesSum = 0;
            dash.marketRateOfChange = 0;
            dash.allCompanyProductRateOfChange = 0;

            // Form dateArray and dataArray, which will be used for chart's labels and dataset accordingly
            var dateArray = dash.timelineArray;

            var profileProductsDataArray = [];
            var profileMarketDataArray = [];

            angular.forEach(dateArray, function(date) {
                // Sales sum for all company products on date X
                var companySalesSum = 0;
                if(allCompanySalesGroupByDate[date.apiFormat]) {
                    var companySales = allCompanySalesGroupByDate[date.apiFormat];
                    angular.forEach(companySales, function(sales) {
                        if(sales.Sales_Data_vod__c) {
                            companySalesSum += parseFloat(sales.Sales_Data_vod__c);
                        }
                    });
                }
                profileProductsDataArray.push(companySalesSum);
                dash.allCompanyProductSalesSum += companySalesSum;

                // Sales sum for all competitor products on date X
                var competitorSalesSum = 0;
                if(allCompetitorSalesGroupByDate[date.apiFormat]) {
                    var competitorSales = allCompetitorSalesGroupByDate[date.apiFormat];
                    angular.forEach(competitorSales, function(sales) {
                        if(sales.Sales_Data_vod__c) {
                            competitorSalesSum += parseFloat(sales.Sales_Data_vod__c);
                        }
                    });
                }
                var marketSalesSum = companySalesSum + competitorSalesSum;
                profileMarketDataArray.push(marketSalesSum);
                dash.marketSalesSum += marketSalesSum;
            });

            chartDataManager.setProfileProductChartData(profileProductsDataArray);
            chartDataManager.setProfileMarketChartData(profileMarketDataArray);

            // Get data for previous period in order to calculate the rate of change between period
            var previousDates = [];
            if(dateArray[0]) {
                previousPeriodStartDate = subtractOneUnitOfTime(dateArray[0].value, dash.selectedPeriod.meta.frequency);
                previousDates = generateTimelineArray(previousPeriodStartDate.value);
            }

            var allCompanyProductLastPeriodSalesSum = 0;
            var marketLastPeriodSalesSum = 0;

            angular.forEach(previousDates, function(pDate) {
                // Sales sum for all company products on date X
                var companyLastPeriodSalesSum = 0;
                if(allCompanySalesGroupByDate[pDate.apiFormat]) {
                    var companySales = allCompanySalesGroupByDate[pDate.apiFormat];
                    angular.forEach(companySales, function(sales) {
                        if(sales.Sales_Data_vod__c) {
                            companyLastPeriodSalesSum += parseFloat(sales.Sales_Data_vod__c);
                        }
                    });
                }
                allCompanyProductLastPeriodSalesSum += companyLastPeriodSalesSum;

                // Sales sum for all competitor products on date X
                var competitorLastPeriodSalesSum = 0;
                if(allCompetitorSalesGroupByDate[pDate.apiFormat]) {
                    var competitorSales = allCompetitorSalesGroupByDate[pDate.apiFormat];
                    angular.forEach(competitorSales, function(sales) {
                        if(sales.Sales_Data_vod__c) {
                            competitorLastPeriodSalesSum += parseFloat(sales.Sales_Data_vod__c);
                        }
                    });
                }
                marketLastPeriodSalesSum += companyLastPeriodSalesSum + competitorLastPeriodSalesSum;
            });

            if(allCompanyProductLastPeriodSalesSum) {
                dash.allCompanyProductRateOfChange = Math.round(100 * 100 * (dash.allCompanyProductSalesSum - allCompanyProductLastPeriodSalesSum) / allCompanyProductLastPeriodSalesSum) / 100;
            } else {
                dash.allCompanyProductRateOfChange = null;
            }

            if(marketLastPeriodSalesSum) {
                dash.marketRateOfChange = Math.round(100 * 100 * (dash.marketSalesSum - marketLastPeriodSalesSum) / marketLastPeriodSalesSum) / 100;
            } else {
                dash.marketRateOfChange = null;
            }
        }

        /**
         * Generate sales table data, including:
         * 1. Calculate aggregated sales value for each product, filtered by the selected period
         * 2. Calculate aggregated sales value for each product, filtered by the previous period
         */
        function salesTableDataProcessor() {
            var previousPeriodStartDate;

            dash.companySalesAggregatedValueByProduct = {};
            dash.competitorSalesAggregatedValueByProduct = {};

            dash.companyPreviousSalesAggregatedValueByProduct = {};
            dash.competitorPreviousSalesAggregatedValueByProduct = {};

            var productAggValByDate = getProductAggregatedValueByDate();
            var dateArray = dash.timelineArray;
            var coProdAggVal, competitorProdAggVal;

            _.values(productAggValByDate).forEach(function(prodAggValByPgId) {
                Object.keys(dash.salesSummaryByCompanyPgId).forEach(function(pgId) {
                    if(!dash.companySalesAggregatedValueByProduct[pgId]) {
                        dash.companySalesAggregatedValueByProduct[pgId] = 0;
                    }
                    coProdAggVal = prodAggValByPgId[pgId] || 0;
                    dash.companySalesAggregatedValueByProduct[pgId] += coProdAggVal;
                });

                Object.keys(dash.salesSummaryByCompetitorPgId).forEach(function(pgId) {
                    if(!dash.competitorSalesAggregatedValueByProduct[pgId]) {
                        dash.competitorSalesAggregatedValueByProduct[pgId] = 0;
                    }
                    competitorProdAggVal = prodAggValByPgId[pgId] || 0;
                    dash.competitorSalesAggregatedValueByProduct[pgId] += competitorProdAggVal;
                });
            });

            // For previous date range
            if(dateArray.length) {
                previousPeriodStartDate = subtractOneUnitOfTime(_.first(dateArray).value, dash.selectedPeriod.meta.frequency);
                var pDateArray = generateTimelineArray(previousPeriodStartDate.value);
                productAggValByDate = getProductAggregatedValueByDate(pDateArray);

                _.values(productAggValByDate).forEach(function(prodAggValByPgId) {
                    Object.keys(dash.salesSummaryByCompanyPgId).forEach(function(pgId) {
                        if(!dash.companyPreviousSalesAggregatedValueByProduct[pgId]) {
                            dash.companyPreviousSalesAggregatedValueByProduct[pgId] = 0;
                        }
                        coProdAggVal = prodAggValByPgId[pgId] || 0;
                        dash.companyPreviousSalesAggregatedValueByProduct[pgId] += coProdAggVal;
                    });

                    Object.keys(dash.salesSummaryByCompetitorPgId).forEach(function(pgId) {
                        if(!dash.competitorPreviousSalesAggregatedValueByProduct[pgId]) {
                            dash.competitorPreviousSalesAggregatedValueByProduct[pgId] = 0;
                        }
                        competitorProdAggVal = prodAggValByPgId[pgId] || 0;
                        dash.competitorPreviousSalesAggregatedValueByProduct[pgId] += competitorProdAggVal;
                    });
                });
            }
        }

        /**
         * Generate activity chart datasets
         */
        function activityChartDataProcessor() {
            var calls = [];
            var emails = [];
            var rollbackFrequency = dash.selectedPeriod.meta.frequency;
            var dateArray = dash.timelineArray;
            var callDataArray = [];
            var emailDataArray = [];

            if(dash.selectedMyProduct && dash.selectedMyProduct.productGroupId) {
                var productId = analyticsDataService.productIdLookup(dash.selectedMyProduct.productGroupId);
                var callActivities = dash.callDetailByProductId[productId];
                var emailActivities = dash.sentEmailsByProductId[productId];

                calls = _.sortBy(callActivities, function(callActivity) {
                    return callActivity.Call2_vod__c.Call_Datetime_vod__c;
                });
                emails = _.sortBy(emailActivities, function(emailActivity) {
                    return emailActivity.Email_Sent_Date_vod__c;
                });
            }

            var callIdx = 0;
            var emailIdx = 0;

            angular.forEach(dateArray, function(date) {
                var salesPeriodBegin = moment.utc(date.apiFormat, YEAR_MONTH_DATE);
                var salesPeriodEnd;

                if(rollbackFrequency === WEEKLY_TEXT) {
                    salesPeriodEnd = salesPeriodBegin.clone().add(1, LOWERCASE_W_TEXT);
                } else if(rollbackFrequency === MONTHLY_TEXT) {
                    salesPeriodEnd = salesPeriodBegin.clone().add(1, UPPERCASE_M_TEXT);
                }

                var callCount = 0;
                for(; callIdx < calls.length; callIdx++) {
                    var callDateLocal = moment.tz(calls[callIdx].Call2_vod__c.Call_Datetime_vod__c, FULL_DATE_TIME_FORMAT, user_timezone).format(YEAR_MONTH_DATE);
                    var callDate = moment.utc(callDateLocal, YEAR_MONTH_DATE);
                    if(!callDate.isValid() || callDate.isBefore(salesPeriodBegin)) {
                        continue;
                    }
                    if(callDate.isBetween(salesPeriodBegin, salesPeriodEnd, null, '[)')) {
                        callCount += 1;
                    } else {
                        break;
                    }
                }
                callDataArray.push(callCount);

                var emailCount = 0;
                for(; emailIdx < emails.length; emailIdx++) {
                    var emailDateLocal = moment.tz(emails[emailIdx].Email_Sent_Date_vod__c, FULL_DATE_TIME_FORMAT, user_timezone).format(YEAR_MONTH_DATE);
                    var emailDate = moment.utc(emailDateLocal, YEAR_MONTH_DATE);
                    if(!emailDate.isValid() || emailDate.isBefore(salesPeriodBegin)) {
                        continue;
                    }
                    if(emailDate.isBetween(salesPeriodBegin, salesPeriodEnd, null, '[)')) {
                        emailCount += 1;
                    } else {
                        break;
                    }
                }
                emailDataArray.push(emailCount);
            });

            chartDataManager.setCallActivityChartData(callDataArray);
            chartDataManager.setEmailActivityChartData(emailDataArray);
        }

        /**
         * Return a sorted date array(start from the latest date, then moving backward) based on the given period start date.
         * @param periodStartDate the start date in 'YYYY-MM-DD' string format
         */
        function generateTimelineArray(periodStartDate) {
            var dates = [];

            if(!periodStartDate) {
                return dates;
            }

            var latestPeriodStartDate = moment.utc(periodStartDate);
            var rollbackValue = dash.selectedPeriod.meta.value;
            var rollbackUnit = dash.selectedPeriod.meta.unit;
            var rollbackFrequency = dash.selectedPeriod.meta.frequency;

            var recordDate = latestPeriodStartDate;
            var firstDate = latestPeriodStartDate.clone().subtract(rollbackValue - 1, rollbackUnit);

            dates.push({
                apiFormat: recordDate.format(YEAR_MONTH_DATE),
                displayFormat: recordDate.format(Three_CHAR_MONTH_DATE_FORMAT),
                value: recordDate.valueOf()
            });

            // Keep rolling back until recordDate go beyond firstDate
            while(firstDate.isBefore(recordDate)) {
                var dateObj = {};
                if(rollbackFrequency === WEEKLY_TEXT) {
                    dateObj.apiFormat = recordDate.subtract(1, LOWERCASE_W_TEXT).format(YEAR_MONTH_DATE);
                } else if(rollbackFrequency === MONTHLY_TEXT) {
                    dateObj.apiFormat = recordDate.subtract(1, UPPERCASE_M_TEXT).format(YEAR_MONTH_DATE);
                }

                dateObj.displayFormat = recordDate.format(Three_CHAR_MONTH_DATE_FORMAT);
                dateObj.value = recordDate.valueOf();
                dates.unshift(dateObj);
            }

            // Return the sorted copy for the date, in ascending order
            return _.sortBy(dates, function(date) {
                return date.value;
            });
        }

        function isMonthlyScale(scale) {
            return scale === MONTHLY_TEXT;
        }

        function isWeeklyScale(scale) {
            return scale === WEEKLY_TEXT;
        }

        function subtractOneUnitOfTime(dateValue, unit) {
            var momentDate = moment.utc(dateValue);
            var dateClone = momentDate.clone();
            var apiFormat;

            if(isWeeklyScale(unit)) {
                apiFormat = dateClone.subtract(1, LOWERCASE_W_TEXT).format(YEAR_MONTH_DATE);
            } else if(isMonthlyScale(unit)) {
                apiFormat = dateClone.subtract(1, UPPERCASE_M_TEXT).format(YEAR_MONTH_DATE);
            }

            return {
                apiFormat: apiFormat,
                displayFormat: dateClone.format(Three_CHAR_MONTH_DATE_FORMAT),
                value: dateClone.valueOf()
            }
        }

        function updateProfileChartDataModel() {
            dash.profileTrendingChart = {
                market: {
                    labels: chartDataManager.getChartLabels(),
                    datasets: [{
                        data: chartDataManager.getProfileMarketChartData(),
                        borderColor: "#ffffff",
                        pointBackgroundColor: "#ffffff"
                    }]
                },
                product: {
                    labels: chartDataManager.getChartLabels(),
                    datasets: [{
                        data: chartDataManager.getProfileProductChartData(),
                        borderColor: "#ffffff",
                        pointBackgroundColor: "#ffffff"
                    }]
                }
            };

            if(dash.noSalesData) {
                dash.profileTrendingChart.backgroundText = {
                    text: appConstant.NO_DATA_AVAILABLE,
                    fontSize: '16px',
                    color: 'rgba(255,255,255,0.6)'
                };
            }
        }

        function updateSalesChartDataModel() {
            dash.productSalesChart = {
                labels: chartDataManager.getChartLabels(),
                datasets: [{
                    data: chartDataManager.getMyProductSalesChartData(),
                    borderColor: '#ff9e16'
                }]
            };

            if(dash.selectedCompetitorProduct && dash.selectedCompetitorProduct.productGroupId) {
                dash.productSalesChart.datasets.push({
                    data: chartDataManager.getCompetitorProductSalesChartData(),
                    borderColor: '#004e7c'
                });
            }

            var noProductData = !chartDataManager.getMyProductSalesChartData().length && !chartDataManager.getCompetitorProductSalesChartData().length;

            if(dash.noSalesData || noProductData) {
                dash.productSalesChart.backgroundText = {
                    text: appConstant.NO_DATA_AVAILABLE,
                    fontSize: '32px'
                };
            }
        }

        function updateActivityChartDataModel() {
            dash.productActivityChart = {
                labels: chartDataManager.getChartLabels(),
                datasets: [{
                    data: [],
                    backgroundColor: '#38cdb8'
                }]
            };

            if(dash.selectedActivityType) {
                if(dash.selectedActivityType.id === 'calls') {
                    dash.productActivityChart.datasets[0].data = chartDataManager.getCallActivityChartData();
                } else if(dash.selectedActivityType.id === 'emails') {
                    dash.productActivityChart.datasets[0].data = chartDataManager.getEmailActivityChartData();
                }
            }

            var noActivityData = false;
            if(dash.selectedActivityType) {
                if(dash.selectedActivityType.id === 'calls') {
                    noActivityData = chartDataManager.getCallActivityChartData().length === 0;
                } else if(dash.selectedActivityType.id === 'emails') {
                    noActivityData = chartDataManager.getEmailActivityChartData().length === 0;
                }
            }

            if(dash.noSalesData || noActivityData) {
                dash.productActivityChart.backgroundText = {
                    text: appConstant.NO_DATA_AVAILABLE,
                    fontSize: '32px'
                };
            }
        }

        function updateSalesTableDataModel() {
            // Clean up
            dash.productSalesTable.data = [];
            var companySalesTableData = [];
            var competitorSalesTableData = [];

            var rollingSum = 0;
            var previousSum = 0;

            // Company products
            var companyProductGroupIds = _.keys(dash.salesSummaryByCompanyPgId);
            angular.forEach(companyProductGroupIds, function(pgId) {
                var rollingValue = dash.companySalesAggregatedValueByProduct[pgId];
                var previousValue = dash.companyPreviousSalesAggregatedValueByProduct[pgId];

                var changePercentage = previousValue ? Math.round(100 * 100 * (rollingValue - previousValue) / previousValue) / 100 : null;
                var productName = dash.analyticsProductGroupSet[pgId] ? dash.analyticsProductGroupSet[pgId].Display_Name_vod__c : '';

                //excluding the parent products
                if(dash.salesSummaryByCompanyPgId[pgId].length > 0) {
                    rollingSum += rollingValue;
                    previousSum += previousValue;
                }

                companySalesTableData.push({
                    product: productName,
                    rollingValue: Math.round(rollingValue),
                    previousValue: Math.round(previousValue),
                    trendingPercentage: {
                        rollingValue: changePercentage
                    },
                    footnoteData: {
                        product: ''
                    }
                });
            });

            dash.productSalesTable.data = _.sortBy(companySalesTableData, function(data) {
                return data.product;
            });

            // Competitor products
            var competitorProductGroupIds = _.keys(dash.salesSummaryByCompetitorPgId);
            angular.forEach(competitorProductGroupIds, function(pgId) {
                var rollingValue = dash.competitorSalesAggregatedValueByProduct[pgId];
                var previousValue = dash.competitorPreviousSalesAggregatedValueByProduct[pgId];

                var changePercentage = previousValue ? Math.round(100 * 100 * (rollingValue - previousValue) / previousValue) / 100 : null;
                var productName = dash.analyticsProductGroupSet[pgId] ? dash.analyticsProductGroupSet[pgId].Display_Name_vod__c : '';

                //excluding the competitor parent products
                if(dash.salesSummaryByCompetitorPgId[pgId].length > 0) {
                    rollingSum += rollingValue;
                    previousSum += previousValue;
                }

                dash.productSalesTable.data.push({
                    product: productName,
                    rollingValue: Math.round(rollingValue),
                    previousValue: Math.round(previousValue),
                    trendingPercentage: {
                        rollingValue: changePercentage
                    },
                    footnoteData: {
                        product: 'Competitor'
                    }
                });
            });

            dash.productSalesTable.data = dash.productSalesTable.data.concat(_.sortBy(competitorSalesTableData, function(data) {
                return data.product;
            }));

            // Add the accumulated value on top
            if(dash.productSalesTable.data.length) {
                dash.productSalesTable.data.unshift({
                    product: 'All',
                    rollingValue: Math.round(rollingSum),
                    previousValue: Math.round(previousSum),
                    trendingPercentage: {
                        rollingValue: previousSum ? Math.round(100 * 100 * (rollingSum - previousSum) / previousSum) / 100 : null
                    },
                    footnoteData: {
                        product: ''
                    }
                });
            }
        }

        function updateActivityTableDataModel() {
            // Clean up
            dash.productActivityTable.data = [];

            var tempActivityTableData = [];
            var totalCallRecord = {};
            var totalCallSampleCount = 0;

            var totalSentEmailRecord = {};
            var dateArray = dash.timelineArray;
            var periodStart = moment.utc();
            var periodEnd = moment.utc();

            var childProducts = findChildProducts();

            if(dateArray.length) {
                periodStart = moment.utc(_.first(dateArray).value);
                if(dash.selectedScale.apiName === 'Month_vod') {
                    periodEnd = moment.utc(_.last(dateArray).value).endOf('month');
                } else if(dash.selectedScale.apiName === 'Week_vod') {
                    periodEnd = moment.utc(_.last(dateArray).value).endOf('week');
                }
            }

            var numberOfWeeks = Math.ceil(periodEnd.diff(periodStart, 'weeks', true)) || 1;

            // Company products
            var companyProductGroupIds = _.keys(dash.salesSummaryByCompanyPgId);
            angular.forEach(companyProductGroupIds, function(pgId) {
                var productName = dash.analyticsProductGroupSet[pgId] ? dash.analyticsProductGroupSet[pgId].Display_Name_vod__c : '';

                var productId = analyticsDataService.productIdLookup(pgId);
                var callDetailActivities = dash.callDetailByProductId[productId];
                var sentEmailActivities = dash.sentEmailsByProductId[productId];

                var callCount = callDetailActivities ? _.size(callDetailActivities) : 0;
                var clmCount = 0;
                angular.forEach(callDetailActivities, function(activity) {
                    totalCallRecord[activity.Call2_vod__c.Id] = activity.Call2_vod__c;
                    if(activityDataService.isClmCall(activity.Call2_vod__c)) {
                        clmCount += 1;
                    }
                });

                var sentEmailCount = sentEmailActivities ? _.size(sentEmailActivities) : 0;
                var openedEmailCount = 0;
                angular.forEach(sentEmailActivities, function(activity) {
                    totalSentEmailRecord[activity.Id] = activity;
                    if(activityDataService.isEmailOpened(activity)) {
                        openedEmailCount += 1;
                    }
                });

                var sampleCount = activityDataService.callSampleRollup(productId, dash.productParentMap, dash.callSampleByProductId);

                totalCallSampleCount += sampleCount;

                if(!childProducts[pgId]){
                    tempActivityTableData.push({
                        product: productName,
                        calls: callCount,
                        callsPerWeek: callCount / numberOfWeeks,
                        sentEmails: sentEmailCount,
                        sentEmailsPerWeek: sentEmailCount / numberOfWeeks,
                        samples: sampleCount,
                        supplementaryData: {
                            calls: callCount ? Math.round(100 * 10 * clmCount / callCount) / 10 : 0,
                            sentEmails: sentEmailCount ? Math.round(100 * 10 * openedEmailCount / sentEmailCount) / 10 : 0
                        }
                    });
                }
            });

            dash.productActivityTable.data = _.sortBy(tempActivityTableData, function(data) {
                return data.product;
            });

            var totalCallCount = _.size(totalCallRecord);
            var totalClmCount = _.reduce(totalCallRecord, function(aggregatedValue, callRecord) {
                if(activityDataService.isClmCall(callRecord)) {
                    return aggregatedValue + 1;
                } else {
                    return aggregatedValue;
                }
            }, 0);

            var totalSentEmailCount = _.size(totalSentEmailRecord);
            var totalOpenedEmailCount = _.reduce(totalSentEmailRecord, function(aggregatedValue, emailRecord) {
                if(activityDataService.isEmailOpened(emailRecord)) {
                    return aggregatedValue + 1;
                } else {
                    return aggregatedValue;
                }
            }, 0);

            // Add the accumulated value on top
            if(dash.productActivityTable.data.length) {
                dash.productActivityTable.data.unshift({
                    product: 'All',
                    calls: totalCallCount,
                    callsPerWeek: totalCallCount / numberOfWeeks,
                    sentEmails: totalSentEmailCount,
                    sentEmailsPerWeek: totalSentEmailCount / numberOfWeeks,
                    samples: totalCallSampleCount,
                    supplementaryData: {
                        calls: totalCallCount ? Math.round(100 * 10 * totalClmCount / totalCallCount) / 10 : 0,
                        sentEmails: totalSentEmailCount ? Math.round(100 * 10 * totalOpenedEmailCount / totalSentEmailCount) / 10 : 0
                    }
                });
            }
        }

        function chartAndTableDataModelBulkUpdate() {
            updateSalesChartDataModel();
            updateProfileChartDataModel();
            initializeSalesTable();
            initializeActivityTable();
        }

        function salesDataCleanUp() {
            dash.salesDataLastUpdatedDate = null;

            dash.analyticsProductGroupSet = {};
            dash.myProductOptions = [];
            dash.competitorProductOptions = [];
            dash.selectedMyProduct = {};
            dash.selectedCompetitorProduct = {};

            dash.salesSummaryByCompanyPgId = {};
            dash.salesSummaryByCompetitorPgId = {};
            dash.companyProductToSalesSummaryMapByDate = {};
            dash.competitorProductToSalesSummaryMapByDate = {};

            // for profile panel
            dash.allCompanyProductSalesSum = 0;
            dash.marketSalesSum = 0;
            dash.allCompanyProductRateOfChange = 0;
            dash.marketRateOfChange = 0;

            // Clean up activity data and update activity chart
            activityDataCleanUp();
            updateActivityChartDataModel();
        }

        function activityDataCleanUp() {
            dash.callDetailByProductId = {};
            dash.callSampleByProductId = {};
            dash.sentEmailsByProductId = {};
        }

        function registerChartPlugin() {
            Chart.pluginService.register(chartJsPlugins.backgroundStrip);
            Chart.pluginService.register(chartJsPlugins.backgroundText);
            Chart.pluginService.register(chartJsPlugins.labelBackground);
            Chart.pluginService.register(chartJsPlugins.alignChart);
        }
    }
})();
