(function() {
    'use strict';

    angular.module('asdApp')
        .directive('barChart', ['$timeout', directiveFn]);

    function directiveFn($timeout) {
        return {
            restrict: 'E',
            replace: true,
            template: '<canvas></canvas>',
            scope: {
                width: '@',
                height: '@',
                chartLabels: '=',
                chartDatasets: '=',
                backgroundText: '='
            },
            link: function(scope, element) {
                setChartSize();

                var yAbsMin = null;
                var yAbsMax = null;
                var yStepSize = null;
                var chartData = {
                    labels: scope.chartLabels,
                    datasets: formDatasets(scope.chartDatasets)
                };
                var chartOptions = {
                    legend: false,
                    events: [],
                    scales: {
                        xAxes: [{
                            categoryPercentage: 1,
                            barPercentage: 0.5,
                            ticks: {
                                display: false
                            }
                        }],
                        yAxes: [{
                            ticks: {
                                min: yAbsMin,
                                max: yAbsMax,
                                stepSize: yStepSize,
                                callback: function(value) {
                                    if(value !== yAbsMin && value !== yAbsMax) {
                                        return value;
                                    }
                                }
                            }
                        }]
                    },
                    alignChart: true,
                    backgroundText: scope.backgroundText
                };

                function setChartSize() {
                    var chartWidth = '100%';
                    var chartHeight = '100%';
                    if(scope.width) {
                        chartWidth = scope.width;
                    }
                    if(scope.height) {
                        chartHeight = scope.height;
                    }
                    element.css({
                        width: chartWidth,
                        height: chartHeight
                    });
                }

                function formDatasets(chartDatasets) {
                    var yMin = null;
                    var yMax = null;

                    // Get the min/max value from the dataset
                    angular.forEach(chartDatasets, function(dataset) {
                        if(yMin === null) {
                            yMin = Math.min.apply(null, dataset.data);
                        } else {
                            yMin = Math.min(yMin, Math.min.apply(null, dataset.data));
                        }

                        if(yMax === null) {
                            yMax = Math.max.apply(null, dataset.data);
                        } else {
                            yMax = Math.max(yMax, Math.max.apply(null, dataset.data));
                        }
                    });

                    // Calculate the stepSize for y-axis and the absolute min/max value for the chart
                    var useableYStep = 3;
                    var preciseStepSize = (yMax - yMin) / useableYStep;
                    var tempStepSize = 1;
                    while(tempStepSize < preciseStepSize) {
                        // check how far away between tempStepSize and preciseStepSize
                        // if it is within the allowance amount, do not increment the tempStepSize anymore
                        var offset = preciseStepSize - tempStepSize;
                        if(offset / tempStepSize >= 0.1) {
                            tempStepSize *= 5; // increment by the base of 5
                        } else {
                            break;
                        }
                    }
                    yStepSize = tempStepSize;
                    var yTempAbsMin = (yMin - yStepSize) >= 0 ? yMin - yStepSize : 0;
                    var yTempAbsMax = yMax + yStepSize;
                    yAbsMin = Math.floor(yTempAbsMin / yStepSize) * yStepSize;
                    yAbsMax = Math.ceil(yTempAbsMax / yStepSize) * yStepSize;

                    return chartDatasets;
                }

                scope.$on('updateBarChart', function() {
                    $timeout(function() {
                        chart.data.labels = scope.chartLabels;
                        chart.data.datasets = formDatasets(scope.chartDatasets);
                        chart.options.backgroundText = scope.backgroundText;
                        chart.options.scales.yAxes[0].ticks.min = yAbsMin;
                        chart.options.scales.yAxes[0].ticks.max = yAbsMax;
                        chart.options.scales.yAxes[0].ticks.stepSize = yStepSize;
                        chart.options.scales.yAxes[0].ticks.callback = function (value) {
                            if (value !== yAbsMin && value !== yAbsMax) {
                                return value;
                            }
                        };
                        chart.update();
                    });
                });

                var chart = new Chart(element, {
                    type: 'bar',
                    data: chartData,
                    options: chartOptions
                });
            }
        };
    }
})();