(function() {
    'use strict';

    angular.module('asdApp')
        .directive('mainLineChart', ['$timeout', 'localeService', directiveFn]);

    function directiveFn($timeout, localeService) {

        return {
            restrict: 'E',
            replace: true,
            template: '<canvas></canvas>',
            scope: {
                width: '@',
                height: '@',
                chartLabels: '=',
                chartDatasets: '=',
                backgroundText: '='
            },
            link: function(scope, element) {

                setChartSize();

                var yAbsMin = null;
                var yAbsMax = null;
                var yStepSize = null;
                var chartData = {
                    labels: scope.chartLabels,
                    datasets: formDatasets(scope.chartDatasets)
                };
                var chartOptions = {
                    legend: false,
                    events: [],
                    scales: {
                        xAxes: [{
                            ticks: {
                                fontStyle: 'bold'
                            }
                        }],
                        yAxes: [{
                            gridLines: {
                                display: false
                            },
                            ticks: {
                                min: yAbsMin,
                                max: yAbsMax,
                                stepSize: yStepSize,
                                callback: function(value) {
                                    if(value !== yAbsMin && value !== yAbsMax) {
                                        return localeService.toLocaleString(value);
                                    }
                                }
                            }
                        }]
                    },
                    // tooltips: {
                    //     enabled: false,
                    //     intersect: false,
                    //     mode: 'index',
                    //     custom: customTooltips,
                    //     filter: function(tooltipItem) {
                    //         if(isHighlightedEvent(tooltipItem.xLabel)) {
                    //             return true;
                    //         } else {
                    //             return false;
                    //         }
                    //     },
                    //     callbacks: {
                    //         title: function(tooltipItems) {
                    //             var tooltipItem = tooltipItems[0];
                    //             if(tooltipItem) {
                    //                 return tooltipItem.xLabel;
                    //             }
                    //         },
                    //         label: function(tooltipItem) {
                    //             if(tooltipItem && tooltipItem.datasetIndex === 0) {
                    //                 return getHighlightedEventMessage(tooltipItem.xLabel);
                    //             }
                    //         }
                    //     }
                    // },
                    backgroundStrip: {
                        color: '#fafafb'
                    },
                    labelBackground: {
                        color: '#edf5f9',
                        borderColor: '#e0e0e0',
                        borderWidth: 1
                    },
                    // highlight: scope.highlightedEvents,
                    alignChart: {
                        primary: true
                    },
                    backgroundText: scope.backgroundText
                };

                function setChartSize() {
                    var chartWidth = '100%';
                    var chartHeight = '100%';
                    if(scope.width) {
                        chartWidth = scope.width;
                    }
                    if(scope.height) {
                        chartHeight = scope.height;
                    }
                    element.css({
                        width: chartWidth,
                        height: chartHeight
                    });
                }

                function formDatasets(chartDatasets) {
                    var defaultChartDataset = {
                        type: 'line',
                        pointRadius: 5,
                        backgroundColor: '#ffffff',
                        borderWidth: 2,
                        fill: false,
                        tension: 0.1
                    };

                    var datasets = [];
                    var yMin = null;
                    var yMax = null;

                    // Get the min/max value from the dataset
                    angular.forEach(chartDatasets, function(dataset) {
                        if(yMin === null) {
                            yMin = Math.min.apply(null, dataset.data);
                        } else {
                            yMin = Math.min(yMin, Math.min.apply(null, dataset.data));
                        }

                        if(yMax === null) {
                            yMax = Math.max.apply(null, dataset.data);
                        } else {
                            yMax = Math.max(yMax, Math.max.apply(null, dataset.data));
                        }

                        datasets.push(_.extend({}, defaultChartDataset, dataset));
                    });

                    // Calculate the stepSize for y-axis and the absolute min/max value for the chart
                    var useableYStep = 5;
                    var preciseStepSize = (yMax - yMin) / useableYStep;
                    var tempStepSize = 1;
                    while(tempStepSize < preciseStepSize) {
                        // check how far away between tempStepSize and preciseStepSize
                        // if it is within the allowance amount, do not increment the tempStepSize anymore
                        var offset = preciseStepSize - tempStepSize;
                        if(offset / tempStepSize >= 0.1) {
                            tempStepSize *= 10; // increment by the base of 10
                        } else {
                            break;
                        }
                    }
                    yStepSize = tempStepSize;
                    var yTempAbsMin = (yMin - yStepSize) >= 0 ? yMin - yStepSize : 0;
                    var yTempAbsMax = yMax + yStepSize;
                    yAbsMin = Math.floor(yTempAbsMin / yStepSize) * yStepSize;
                    yAbsMax = Math.ceil(yTempAbsMax / yStepSize) * yStepSize;

                    // Create a dummy bar chart dataset so that the line chart data point will align in the center of each x'tick
                    var dummyData = [];
                    if(datasets[0]) {
                        for(var i = 0; i < datasets[0].data.length; i++) {
                            dummyData.push(yAbsMin);
                        }
                    }
                    datasets.push(_.extend({}, defaultChartDataset), {
                        type: 'bar',
                        data: dummyData
                    });

                    return datasets;
                }

                // function customTooltips(tooltip) {
                //     // Tooltip Element
                //     var tooltipEl = document.getElementById('chartjs-tooltip');
                //
                //     if (!tooltipEl) {
                //         tooltipEl = document.createElement('div');
                //         tooltipEl.id = 'chartjs-tooltip';
                //         tooltipEl.innerHTML = "<table></table>";
                //         document.body.appendChild(tooltipEl);
                //     }
                //
                //     // Hide if no tooltip
                //     if (tooltip.opacity === 0 || !tooltip.body || !tooltip.body.length) {
                //         tooltipEl.style.opacity = 0;
                //         return;
                //     }
                //
                //     // Set Text
                //     if (tooltip.body) {
                //         var titleLines = tooltip.title || [];
                //         var bodyLines = tooltip.body.map(function(bodyItem) {
                //             return bodyItem.lines;
                //         });
                //
                //         var innerHtml = '<thead>';
                //
                //         titleLines.forEach(function(title) {
                //             innerHtml += '<tr><th>' + title + '</th></tr>';
                //         });
                //         innerHtml += '</thead><tbody>';
                //
                //         bodyLines.forEach(function(body) {
                //             body.forEach(function(line) {
                //                 innerHtml += '<tr><td>' + line + '</td></tr>';
                //             });
                //         });
                //         innerHtml += '</tbody>';
                //
                //         var tableRoot = tooltipEl.querySelector('table');
                //         tableRoot.innerHTML = innerHtml;
                //     }
                //
                //     // Display, position, and set styles for font
                //     var position = this._chart.canvas.getBoundingClientRect();
                //     var arrowSize = tooltip.caretSize + 10;
                //     var absYPos = position.top;
                //     if(!isNaN(parseInt(tooltipEl.style.top))) {
                //         absYPos = Math.max(parseInt(tooltipEl.style.top), absYPos);
                //     }
                //
                //     tooltipEl.classList.remove('point-left', 'point-right');
                //     if(tooltip.caretX + arrowSize + tooltip.width <= position.right) {
                //         tooltipEl.style.left = position.left + tooltip.caretX + arrowSize + 'px';
                //         tooltipEl.classList.add('point-left');
                //     } else {
                //         tooltipEl.style.left = position.left + tooltip.caretX - arrowSize - tooltip.width + 'px';
                //         tooltipEl.classList.add('point-right');
                //     }
                //
                //     tooltipEl.style.top = absYPos + 'px';
                //     tooltipEl.style.opacity = 1;
                //     tooltipEl.style.padding = tooltip.yPadding + 'px ' + tooltip.xPadding + 'px';
                // }

                // function isHighlightedEvent(label) {
                //     return angular.isDefined(_.find(scope.highlightedEvents, function(highlightedEvent) {
                //         return highlightedEvent.date === label;
                //     }));
                // }

                // function getHighlightedEventMessage(label) {
                //     var event = _.find(scope.highlightedEvents, function(highlightedEvent) {
                //         return highlightedEvent.date === label;
                //     });
                //
                //     if(event) {
                //         return event.message;
                //     }
                // }

                scope.$on('updateMainLineChart', function() {
                    // Wait for DOM to finish rendering so that the two-way binding value is updated first
                    $timeout(function() {
                        chart.data.labels = scope.chartLabels;
                        chart.data.datasets = formDatasets(scope.chartDatasets);
                        // chart.options.highlight = scope.highlightedEvents;
                        chart.options.backgroundText = scope.backgroundText;
                        chart.options.scales.yAxes[0].ticks.min = yAbsMin;
                        chart.options.scales.yAxes[0].ticks.max = yAbsMax;
                        chart.options.scales.yAxes[0].ticks.stepSize = yStepSize;
                        chart.options.scales.yAxes[0].ticks.callback = function (value) {
                            if (value !== yAbsMin && value !== yAbsMax) {
                                return localeService.toLocaleString(value);
                            }
                        };
                        chart.update();
                    });
                });

                var chart = new Chart(element, {
                    type: 'bar',
                    data: chartData,
                    options: chartOptions
                });
            }
        };
    }
})();