(function() {
    'use strict';

    angular.module('asdApp')
        .directive('profileLineChart', ['$timeout', directiveFn]);

    function directiveFn($timeout) {

        return {
            restrict: 'E',
            replace: true,
            template: '<canvas></canvas>',
            scope: {
                width: '@',
                height: '@',
                chartLabels: '=',
                chartDatasets: '=',
                backgroundText: '='
            },
            link: function(scope, element) {

                setChartSize();

                var gradient = element[0].getContext("2d").createLinearGradient(0, 0, 0, element[0].height);
                gradient.addColorStop(0, 'rgba(255, 255, 255, 0.5)');
                gradient.addColorStop(1, 'rgba(255, 255, 255, 0)');

                var chartData = {
                    labels: scope.chartLabels,
                    datasets: formDatasets(scope.chartDatasets)
                };

                var chartOptions = {
                    responsive: true,
                    legend: false,
                    layout: {
                        padding: 4
                    },
                    events: [],
                    scales: {
                        xAxes: [{
                            display: false
                        }],
                        yAxes: [{
                            display: false
                        }]
                    },
                    backgroundText: scope.backgroundText
                };

                function setChartSize() {
                    var chartWidth = '100%';
                    var chartHeight = '100%';
                    if(scope.width) {
                        chartWidth = scope.width;
                    }
                    if(scope.height) {
                        chartHeight = scope.height;
                    }
                    element.css({
                        width: chartWidth,
                        height: chartHeight
                    });
                }

                function formDatasets(chartDatasets) {
                    var defaultChartDataset = {
                        type: 'line',
                        pointBackgroundColor: '#ffffff',
                        backgroundColor: gradient,
                        borderColor: "#ffffff",
                        borderWidth: 2,
                        tension: 0.1
                    };
                    var datasets = [];

                    angular.forEach(chartDatasets, function(dataset) {
                        var pointRadius = [];
                        for(var i = 0; i < dataset.data.length; i++) {
                            pointRadius.push(0);
                        }
                        if(pointRadius.length) {
                            pointRadius[pointRadius.length - 1] = 2; // Only draw the dot on the last data point
                        }

                        datasets.push(_.extend({}, defaultChartDataset, {pointRadius: pointRadius}, dataset));
                    });

                    return datasets;
                }

                scope.$on('updateProfileChart', function() {
                    $timeout(function() {
                        chart.data.labels = scope.chartLabels;
                        chart.data.datasets = formDatasets(scope.chartDatasets);
                        chart.options.backgroundText = scope.backgroundText;
                        chart.update();
                    });
                });

                var chart = new Chart(element, {
                    type: 'line',
                    data: chartData,
                    options: chartOptions
                });
            }
        };
    }
})();