(function() {
    'use strict';

    angular.module('asdApp')
        .directive('profileSummary', directiveFn);

    function directiveFn() {
        var controller = ['$scope', function($scope) {
            var profile = this;
            var account = $scope.account;

            profile.getAccountName = function() {
                if(account) {
                    if(account.LastName && account.FirstName) {
                        return account.LastName + ', ' + account.FirstName;
                    } else if(account.FirstName) {
                        return account.FirstName;
                    } else {
                        return account.Name;
                    }
                }
                return '';
            };

            profile.getDepartment = function() {
                if(account) {
                    return account.Specialty_1_vod__c || '';
                }
                return '';
            };

            profile.applyChangeStyle = function(rateOfChange) {
                if(rateOfChange) {
                    if(parseFloat(rateOfChange) > 0) {
                        return 'positive';
                    } else {
                        return 'negative';
                    }
                }
            };

            profile.applyChangeIconStyle = function(rateOfChange) {
                if(rateOfChange) {
                    if(parseFloat(rateOfChange) > 0) {
                        return 'fa-arrow-up';
                    } else {
                        return 'fa-arrow-down';
                    }
                }
            };

            profile.formatSalesSum = function(salesSum) {
                if(!isNaN(parseFloat(salesSum))) {
                    return Math.round(parseFloat(salesSum));
                } else {
                    return '--';
                }
            };

            profile.formatRateOfChange = function(rateOfChange) {
                if(!isNaN(parseFloat(rateOfChange))) {
                    return Math.round(parseFloat(rateOfChange)) + '%';
                } else {
                    return '--';
                }
            };
        }];

        return {
            templateUrl: 'views/profile-summary.html',
            restrict: 'E',
            scope: {
                account: '=',
                marketChartLabels: '=',
                marketChartDatasets: '=',
                productChartLabels: '=',
                productChartDatasets: '=',
                selectedDataType: '=',
                marketSum: '=',
                allProductSum: '=',
                marketRateOfChange: '=',
                allProductRateOfChange: '=',
                backgroundText: '='
            },
            controller: controller,
            controllerAs: 'profile'
        };
    }
})();