(function() {
    'use strict';

    angular.module('asdApp')
        .factory('accountSalesSummaryService', ['$q', 'endpoints', factoryFn])
    ;

    function factoryFn($q, endpoints) {
        var assService = this;
        var dataDate;
        var analyticsFiles;
        assService.getAccountSalesSummary = function(scale, dataType, marketId, accountId) {
            var deferred = $q.defer();

            endpoints.fetchAccountSalesSummary(scale, dataType, marketId, accountId).then(function(resp) {
                console.log(resp);
                fetchSalesDataDate(scale, dataType).then(function(date) {
                    dataDate = date;
                    deferred.resolve(resp.Account_Sales_Summary_Insights);
                });
            }, function(error) {
                deferred.reject(error);
            });

            return deferred.promise;
        };

        assService.getSalesDataDate = function() {
            return dataDate;
        };

        assService.getFormattedSalesDataDate = function() {
            if(angular.isDefined(dataDate)) {
                return moment(dataDate, 'YYYY-MM-DD').format('MMM D, YYYY');
            } else {
                return '';
            }
        };

        assService.fetchAnalyticsFilesReport = function() {
            var deferred = $q.defer();
            if(angular.isDefined(analyticsFiles)) {
                deferred.resolve(analyticsFiles);
            } else {
                endpoints.fetchAnalyticsFilesReport().then(function (resp2) {
                    console.log(resp2);
                    analyticsFiles = resp2;
                    deferred.resolve(analyticsFiles);
                });
            }
            return deferred.promise;
        };

        function fetchSalesDataDate(scale, dataType) {
            var deferred = $q.defer();

            endpoints.fetchAnalyticsFiles(scale, dataType).then(function(resp) {
                console.log(resp);
                var rawDate = null;

                if(resp.Analytics_Files_vod__c && resp.Analytics_Files_vod__c.length) {
                    // the analytics files record is already sorted in descending order by last modified date
                    rawDate = resp.Analytics_Files_vod__c[0].Data_Date_vod__c;
                }

                deferred.resolve(rawDate);
            });

            return deferred.promise;
        }

        return assService;
    }
})();