(function() {
    'use strict';

    angular.module('asdApp')
        .factory('activityDataService', ['$q', 'endpoints', factoryFn])
    ;

    function factoryFn($q, endpoints) {
        var activityDataService = this;
        var callsCache = {};

        activityDataService.getCalls = function(accountId, startDate, endDate) {
            var deferred = $q.defer();
            var callsById = {};

            if(callsCache[accountId + startDate + endDate]) {
                callsById = callsCache[accountId + startDate + endDate];
                deferred.resolve(callsById);
            } else {
                endpoints.fetchCalls(accountId, startDate, endDate).then(function(resp1) {
                    console.log(resp1);
                    callsById = _.indexBy(resp1.Call2_vod__c, function(call) {
                        return call.Id;
                    });
                    callsCache[accountId + startDate + endDate] = callsById;
                    deferred.resolve(callsById);
                });
            }

            return deferred.promise;
        };

        activityDataService.getCallDetailByProductId = function(accountId, startDate, endDate) {
            var deferred = $q.defer();

            activityDataService.getCalls(accountId, startDate, endDate).then(function(callsById) {
                var callIds = _.keys(callsById);

                endpoints.fetchCallDetails(callIds).then(function(resp) {
                    console.log(resp);
                    var callDetails = resp.Call2_Detail_vod__c;

                    // Merge call data into call detail data
                    angular.forEach(callDetails, function(callDetail) {
                        if(callDetail.Call2_vod__c && callsById[callDetail.Call2_vod__c]) {
                            callDetail.Call2_vod__c = callsById[callDetail.Call2_vod__c];
                        }
                    });

                    var callDetailByProductId = _.groupBy(callDetails, function(callDetail) {
                        return callDetail.Product_vod__c;
                    });

                    deferred.resolve(callDetailByProductId);
                });
            });

            return deferred.promise;
        };

        activityDataService.getCallSampleByProductId = function(accountId, startDate, endDate) {
            var deferred = $q.defer();

            activityDataService.getCalls(accountId, startDate, endDate).then(function(callsById) {
                var callIds = _.keys(callsById);

                endpoints.fetchCallSamples(callIds).then(function(resp) {
                    console.log(resp);
                    var callSamples = resp.Call2_Sample_vod__c;
                    var callSampleByProductId = _.groupBy(callSamples, function(callSample) {
                        return callSample.Product_vod__c;
                    });

                    deferred.resolve(callSampleByProductId);
                });
            });

            return deferred.promise;
        };

        activityDataService.getSentEmailActivityByProductId = function(accountId, startDate, endDate) {
            var deferred = $q.defer();

            endpoints.fetchSentEmails(accountId, startDate, endDate).then(function(resp) {
                console.log(resp);
                var sentEmailsByProductId = _.groupBy(resp.Sent_Email_vod__c, function(sentEmail) {
                    return sentEmail.Product_vod__c;
                });

                deferred.resolve(sentEmailsByProductId);
            });

            return deferred.promise;
        };

        function isTruthyInAllPlatforms(fieldValue) {
            return fieldValue === 1 || fieldValue === '1' || fieldValue === true || fieldValue === 'true';
        }

        activityDataService.isClmCall = function(call) {
            return !!(call && call.CLM_vod__c && isTruthyInAllPlatforms(call.CLM_vod__c));
        };

        activityDataService.isEmailOpened = function(sentEmail) {
            return !!(sentEmail && sentEmail.Opened_vod__c && isTruthyInAllPlatforms(sentEmail.Opened_vod__c));
        };

        activityDataService.getProductParentMap = function() {
            var productGroupByParent;
            var deferred = $q.defer();

            endpoints.fetchProducts().then(function(resp) {

                // to avoid '' as a key in the output
                _.forEach(resp.Product_vod__c, function(product){
                   if (product.Parent_Product_vod__c === ''){
                       product.Parent_Product_vod__c = 'NoParent';
                   }
                });

                productGroupByParent = _.groupBy(resp.Product_vod__c, 'Parent_Product_vod__c');

                deferred.resolve(productGroupByParent);
            });

            return deferred.promise;
        };

        activityDataService.callSampleRollup = function(productId, parentMapping, callSampleByProductId) {
            var totalCallSample = 0;

            // if this is a parent product
            if (parentMapping[productId]) {

                // sum all sample under this parent
                _.forEach(parentMapping[productId], function(childProduct){
                    if (callSampleByProductId[childProduct.Id]) {
                        _.forEach(callSampleByProductId[childProduct.Id], function(singleProduct){
                            totalCallSample += parseFloat(singleProduct.Quantity_vod__c);
                        });
                    }
                });
            }

            // sum the sample record against the parent itself as well or this is a child product
            if (callSampleByProductId[productId]) {
                _.forEach(callSampleByProductId[productId], function(singleProduct){
                    totalCallSample += parseFloat(singleProduct.Quantity_vod__c);
                });
            }

            return totalCallSample;
        };

        return activityDataService;
    }
})();
