(function() {
    'use strict';

    angular.module('asdApp')
        .factory('analyticsDataService', ['$q', 'endpoints', factoryFn])
    ;

    function factoryFn($q, endpoints) {
        var analyticsDataService = this;
        var analyticsMarkets;
        var myProductGroupMap;
        var myAnalyticsProductGroupIds;
        var productList;

        analyticsDataService.getAnalyticsMarkets = function() {
            var deferred = $q.defer();

            if(angular.isDefined(analyticsMarkets)) {
                deferred.resolve(analyticsMarkets);

            } else {
                // Fetch Products from My Setup Product
                endpoints.fetchMySetupProducts().then(function(resp1) {
                    console.log(resp1);
                    var productTracker = {};
                    var productIds = [];

                    angular.forEach(resp1.My_Setup_Products_vod__c, function(mySetupProd) {
                        var productId = mySetupProd.Product_vod__c;
                        if(productId && productId !== '' && !productTracker[productId]) {
                            productIds.push(productId);
                            productTracker[productId] = true;
                        }
                    });

                    // Fetch Analytics Product Groups from Product Group Map based on Product Ids
                    endpoints.fetchProductGroupMap(productIds).then(function(resp2) {
                        console.log(resp2);
                        var analyticsProductGroupTracker = {};
                        var analyticsProductGroupIds = [];
                        myProductGroupMap = resp2.Product_Group_Map_vod__c;

                        angular.forEach(myProductGroupMap, function(prodGroupMap) {
                            var analyticsProductGroupId = prodGroupMap.Analytics_Product_Group_vod__c;
                            if(analyticsProductGroupId && analyticsProductGroupId !== '' && !analyticsProductGroupTracker[analyticsProductGroupId]) {
                                analyticsProductGroupIds.push(analyticsProductGroupId);
                                analyticsProductGroupTracker[analyticsProductGroupId] = true;
                            }
                        });
                        myAnalyticsProductGroupIds = analyticsProductGroupIds;

                        // Fetch Markets from Analytics Product Group based on Analytics Product Group Ids
                        endpoints.fetchAnalyticsProductGroups(myAnalyticsProductGroupIds).then(function(resp3) {
                            console.log(resp3);
                            var analyticsMarketTracker = {};
                            var analyticsMarketIds = [];

                            angular.forEach(resp3.Analytics_Product_Group_vod__c, function(analyticsProductGroup) {
                                var analyticsMarketId = analyticsProductGroup.Market_vod__c;
                                if(analyticsMarketId && analyticsMarketId !== '' && !analyticsMarketTracker[analyticsMarketId]) {
                                    analyticsMarketIds.push(analyticsMarketId);
                                    analyticsMarketTracker[analyticsMarketId] = true;
                                }
                            });

                            // Fetch Analytics Market metadata from Analytics Market based on Analytics Market Ids
                            endpoints.fetchAnalyticsMarkets(analyticsMarketIds).then(function(resp4) {
                                console.log(resp4);
                                analyticsMarkets = resp4.Analytics_Markets_vod__c;
                                deferred.resolve(analyticsMarkets);
                            });
                        });
                    });
                });
            }

            return deferred.promise;
        };

        analyticsDataService.getProductList = function(marketId) {
            var deferred = $q.defer();

            if(angular.isDefined(productList && productList[marketId])) {
                deferred.resolve(productList[marketId]);

            } else {
                endpoints.fetchAnalyticsProductGroupsForMarket(marketId).then(function (resp) {
                    console.log(resp);
                    if(!productList) {
                        productList = {};
                    }
                    productList[marketId] = _.filter(resp.Analytics_Product_Group_vod__c, function(analyticsProductGroup) {
                        return !isHiddenAnalyticsProductGroup(analyticsProductGroup);
                    });
                    deferred.resolve(productList[marketId]);
                });
            }

            return deferred.promise;
        };

        analyticsDataService.isCompanyAnalyticsProductGroup = function(analyticsProductGroup) {
            if(analyticsProductGroup && analyticsProductGroup.Company_Product_Group_vod__c &&
                (analyticsProductGroup.Company_Product_Group_vod__c === 'True' ||
                analyticsProductGroup.Company_Product_Group_vod__c === true ||
                analyticsProductGroup.Company_Product_Group_vod__c === 1)) {
                return true;
            } else {
                return false;
            }
        };

        analyticsDataService.productIdLookup = function(analyticsProductGroupId) {
            var found = _.find(myProductGroupMap, function(row) {
                return row.Analytics_Product_Group_vod__c === analyticsProductGroupId;
            });

            if(found) {
                return found.Product_vod__c;
            } else {
                return null;
            }
        };

        analyticsDataService.myAnalyticsProductGroupIds = function() {
            return myAnalyticsProductGroupIds;
        };

        analyticsDataService.indexSalesSummaryByDate = function(salesSummary) {
            return _.indexBy(salesSummary, function(row) {
                return row.Sales_Date_vod__c;
            });
        };

        analyticsDataService.groupSalesSummaryByDate = function(salesSummary) {
            return _.groupBy(salesSummary, function(row) {
                return row.Sales_Date_vod__c;
            });
        };

        function isHiddenAnalyticsProductGroup(analyticsProductGroup) {
            if(analyticsProductGroup && analyticsProductGroup.Display_In_Reports_vod__c &&
                (analyticsProductGroup.Display_In_Reports_vod__c === 'True' ||
                analyticsProductGroup.Display_In_Reports_vod__c === true ||
                analyticsProductGroup.Display_In_Reports_vod__c === 1)) {
                return false;
            } else {
                return true;
            }
        }

        return analyticsDataService;
    }
})();