(function() {
    'use strict';

    angular.module('asdApp')
        .factory('endpoints', ['$q', 'queries', factoryFn])
    ;

    function factoryFn($q, queries) {
        var endpoints = this;
        var utils = new window.VeevaUtilities();

        var normalizeWhereClauseBooleanValue = function(value) {
            if(!utils.isOnline() && utils.isWin8()) {
                return (typeof value === 'boolean' && value) ? '\'true\'' : '\'false\'';
            }
            return typeof value === 'boolean' && value;
        };

        endpoints.fetchCurrentUser = function() {
            var deferred = $q.defer();

            ds.getDataForCurrentObject('User', 'Id').then(function(resp1) {
                if(resp1.success) {
                    var currentUser = resp1.User;
                    var queryConfig = angular.copy(queries.user);

                    queryConfig.where = queryConfig.where.replace(/_userId/, '\'' + currentUser.Id + '\'');

                    ds.queryRecord(queryConfig).then(function (resp2) {
                        deferred.resolve(resp2);
                    }, genericQueryErrorHandler(deferred));

                } else {
                    (genericQueryErrorHandler(deferred))(resp1.message);
                }
            });

            return deferred.promise;
        };

        endpoints.fetchCurrentAccount = function() {
            var deferred = $q.defer();

            ds.getDataForCurrentObject('Account', 'Id').then(function(resp) {
                if(resp.success) {
                    deferred.resolve(resp.Account);
                } else {
                    (genericQueryErrorHandler(deferred))(resp.message);
                }
            });

            return deferred.promise;
        };

        endpoints.fetchAccount = function(accountId) {
            var deferred = $q.defer();
            var queryConfig = angular.copy(queries.account);
            queryConfig.where = queryConfig.where.replace(/_accountId/, '\'' + accountId + '\'');

            ds.queryRecord(queryConfig).then(function(resp) {
                deferred.resolve(resp);
            }, genericQueryErrorHandler(deferred));

            return deferred.promise;
        };

        endpoints.fetchMySetupProducts = function() {
            var deferred = $q.defer();
            var queryConfig = angular.copy(queries.mySetupProduct);

            ds.queryRecord(queryConfig).then(function(resp) {
                deferred.resolve(resp);
            }, genericQueryErrorHandler(deferred));

            return deferred.promise;
        };

        endpoints.fetchAnalyticsFilesReport = function() {
            var deferred = $q.defer();
            var queryConfig = angular.copy(queries.analyticsFilesReport);

            ds.queryRecord(queryConfig).then(function(resp) {
                deferred.resolve(resp.Analytics_Files_vod__c);
            }, genericQueryErrorHandler(deferred));

            return deferred.promise;
        };

        endpoints.fetchProductGroupMap = function(productIds) {
            var deferred = $q.defer();
            var queryConfig = angular.copy(queries.productGroupMap);

            if(angular.isDefined(productIds)) {
                queryConfig.where += ds.getInStatement(productIds);
            } else {
                queryConfig.where = null;
            }

            ds.queryRecord(queryConfig).then(function(resp) {
                deferred.resolve(resp);
            }, genericQueryErrorHandler(deferred));

            return deferred.promise;
        };

        endpoints.fetchAnalyticsFiles = function(scale, dataType) {
            var deferred = $q.defer();
            var queryConfig = angular.copy(queries.analyticsFiles);
            queryConfig.where = queryConfig.where.replace(/_false/g, normalizeWhereClauseBooleanValue(false));
            queryConfig.where = queryConfig.where.replace(/_scale/, '\'' + scale + '\'');
            queryConfig.where = utils.replaceStringInWhereClause(queryConfig.where, dataType, '_dataType');

            ds.queryRecord(queryConfig).then(function(resp) {
                deferred.resolve(resp);
            }, genericQueryErrorHandler(deferred));

            return deferred.promise;
        };

        endpoints.fetchAnalyticsProductGroups = function(analyticsProductGroupIds) {
            var deferred = $q.defer();
            var queryConfig = angular.copy(queries.analyticsProductGroup);

            if(angular.isDefined(analyticsProductGroupIds)) {
                queryConfig.where += ds.getInStatement(analyticsProductGroupIds);
            } else {
                queryConfig.where = null;
            }

            ds.queryRecord(queryConfig).then(function(resp) {
                deferred.resolve(resp);
            }, genericQueryErrorHandler(deferred));

            return deferred.promise;
        };

        endpoints.fetchAnalyticsProductGroupsForMarket = function(marketId) {
            var deferred = $q.defer();
            var queryConfig = angular.copy(queries.analyticsProductGroupForMarket);
            queryConfig.where = queryConfig.where.replace(/_marketId/, '\'' + marketId + '\'');

            ds.queryRecord(queryConfig).then(function(resp) {
                deferred.resolve(resp);
            }, genericQueryErrorHandler(deferred));

            return deferred.promise;
        };

        endpoints.fetchAnalyticsMarkets = function(analyticsMarketIds) {
            var deferred = $q.defer();
            var queryConfig = angular.copy(queries.analyticsMarket);
            queryConfig.where += ds.getInStatement(analyticsMarketIds);

            ds.queryRecord(queryConfig).then(function(resp) {
                deferred.resolve(resp);
            }, genericQueryErrorHandler(deferred));

            return deferred.promise;
        };

        endpoints.fetchDataMapTemplates = function() {
            var deferred = $q.defer();
            var queryConfig = angular.copy(queries.dataMapTemplate);
            queryConfig.where = null;

            ds.queryRecord(queryConfig).then(function(resp) {
                deferred.resolve(resp);
            }, genericQueryErrorHandler(deferred));

            return deferred.promise;
        };

        endpoints.fetchAccountSalesSummary = function(scale, dataType, marketId, accountId) {
            var deferred = $q.defer();
            var queryConfig = angular.copy(queries.accountSalesSummary);
            queryConfig.where = queryConfig.where.replace(/_scale/, '\'' + scale + '\'');
            queryConfig.where = utils.replaceStringInWhereClause(queryConfig.where, dataType, '_dataType');
            queryConfig.where = queryConfig.where.replace(/_marketId/, '\'' + marketId + '\'');
            queryConfig.where = queryConfig.where.replace(/_accountId/, '\'' + accountId + '\'');

            ds.querySalesData(queryConfig).then(function(resp) {
                deferred.resolve(resp);
            }, genericQueryErrorHandler(deferred));

            return deferred.promise;
        };

        endpoints.fetchCalls = function(accountId, startDate, endDate) {
            var deferred = $q.defer();
            var queryConfig = angular.copy(queries.call);
            queryConfig.where = queryConfig.where.replace(/_accountId/, '\'' + accountId + '\'');

            if(utils.isOnline()) {
                queryConfig.where = queryConfig.where.replace(/_dateTime1/, startDate);
                queryConfig.where = queryConfig.where.replace(/_dateTime2/, endDate);
            } else {
                queryConfig.where = queryConfig.where.replace(/_dateTime1/, '\'' + startDate + '\'');
                queryConfig.where = queryConfig.where.replace(/_dateTime2/, '\'' + endDate + '\'');
            }

            ds.queryRecord(queryConfig).then(function(resp) {
                deferred.resolve(resp);
            }, genericQueryErrorHandler(deferred));

            return deferred.promise;
        };

        endpoints.fetchCallDetails = function(callIds) {
            var deferred = $q.defer();
            var queryConfig = angular.copy(queries.callDetail);
            queryConfig.where += ds.getInStatement(callIds);

            ds.queryRecord(queryConfig).then(function(resp) {
                deferred.resolve(resp);
            }, genericQueryErrorHandler(deferred));

            return deferred.promise;
        };

        endpoints.fetchCallSamples = function(callIds) {
            var deferred = $q.defer();
            var queryConfig = angular.copy(queries.callSample);
            queryConfig.where += ds.getInStatement(callIds);

            ds.queryRecord(queryConfig).then(function(resp) {
                deferred.resolve(resp);
            }, genericQueryErrorHandler(deferred));

            return deferred.promise;
        };

        endpoints.fetchSentEmails = function(accountId, startDate, endDate) {
            var deferred = $q.defer();
            var queryConfig = angular.copy(queries.sentEmail);
            queryConfig.where = queryConfig.where.replace(/_accountId/, '\'' + accountId + '\'');

            if(utils.isOnline()) {
                queryConfig.where = queryConfig.where.replace(/_dateTime1/, startDate);
                queryConfig.where = queryConfig.where.replace(/_dateTime2/, endDate);
            } else {
                queryConfig.where = queryConfig.where.replace(/_dateTime1/, '\'' + startDate + '\'');
                queryConfig.where = queryConfig.where.replace(/_dateTime2/, '\'' + endDate + '\'');
            }

            ds.queryRecord(queryConfig).then(function(resp) {
                deferred.resolve(resp);
            }, genericQueryErrorHandler(deferred));

            return deferred.promise;
        };

        endpoints.fetchTranslations = function(msgsToGet, userLocale) {
            var deferred = $q.defer();

            ds.getVeevaMessagesWithDefault(msgsToGet, userLocale).then(function (resp) {
                deferred.resolve(resp);
            });

            return deferred.promise;
        };

        endpoints.fetchPicklistLabels = function (objectName, fieldName) {
            var deferred = $q.defer();

            ds.getPicklistValueLabels(objectName, fieldName).then(function(resp) {
                deferred.resolve(resp);
            });

            return deferred.promise;
        };

        endpoints.fetchProducts = function() {
            var deferred = $q.defer();
            var queryConfig = angular.copy(queries.product);

            ds.queryRecord(queryConfig).then(function(resp) {
                deferred.resolve(resp);
            }, genericQueryErrorHandler(deferred));

            return deferred.promise;
        };

        /**
         * Private functions
         */
        function genericQueryErrorHandler(deferred) {
            return function(error) {
                deferred.reject(error);
            };
        }

        return endpoints;
    }
})();
