(function() {
    'use strict';

    angular.module('asdApp')
        .factory('messageService', ['$q', 'endpoints', 'VEEVA_MESSAGES', factoryFn])
    ;

    function factoryFn($q, endpoints, VEEVA_MESSAGES) {
        var messageService = this;
        var translatedMessages = {};

        messageService.getTranslations = function(msgsToGet, userLocale) {
            var deferred = $q.defer();

            endpoints.fetchTranslations(msgsToGet, userLocale).then(function(resp) {
                setupTranslations(resp);
                deferred.resolve(translatedMessages);
            });

            return deferred.promise;
        };

        messageService.getMessage = function() {
            var key = arguments[0];
            if(translatedMessages[key]) {
                var message = translatedMessages[key];
                for(var i = 1; i < arguments.length; i++) {
                    var token = arguments[i];
                    var marker = i - 1;
                    message = message.replace('{' + marker + '}', token);
                }
                return message;
            } else {
                return key;
            }
        };

        messageService.getPicklistLabels = function(objectName, fieldName) {
            var labels = [];
            endpoints.fetchPicklistLabels(objectName, fieldName).then(function(resp) {
                for (var apiName in resp[objectName][fieldName]) {
                    var name = resp[objectName][fieldName][apiName];
                    labels.push({'name': name, 'apiName': apiName});
                }
            });
            return labels;
        };

        function setupTranslations(rawTranslations) {
            angular.forEach(VEEVA_MESSAGES, function(msg, key) {
                var foundTranslation = _.find(rawTranslations, function(rawMsg) {
                    return msg.msgName === rawMsg.Name.value && msg.msgCategory === rawMsg.Category_vod__c.value;
                });

                if(foundTranslation) {
                    translatedMessages[key] = foundTranslation.Text_vod__c.display;
                }
            });
        }

        return messageService;
    }
})();