(function() {
    'use strict';

    angular.module('asdApp')
        .value('queries', {
            user: {
                object: 'User',
                fields: ['Id', 'LocaleSidKey', 'LanguageLocaleKey', 'TimeZoneSidKey'],
                where: 'Id = _userId'
            },
            account: {
                object: 'Account',
                fields: ['Id', 'Specialty_1_vod__c', 'FirstName', 'LastName', 'Name'],
                where: 'Id = _accountId'
            },
            dataMapTemplate: {
                object: 'Data_Map_Template_vod__c',
                fields: ['Id', 'Scale_vod__c', 'Column_Label_vod__c', 'Second_Column_Label_vod__c'],
                where: 'Contains_Goals_vod__c = _false AND Scale_vod__c = _scale AND (Column_Label_vod__c = _dataType OR Second_Column_Label_vod__c = _dataType)'
            },
            mySetupProduct: {
                object: 'My_Setup_Products_vod__c',
                fields: ['Product_vod__c']
            },
            product:{
                object: 'Product_vod__c',
                fields: ['Id', 'Parent_Product_vod__c']
            },
            productGroupMap: {
                object: 'Product_Group_Map_vod__c',
                fields: ['Analytics_Product_Group_vod__c', 'Product_vod__c'],
                where: 'Product_vod__c IN '
            },
            analyticsFiles: {
                object: 'Analytics_Files_vod__c',
                fields: ['Id', 'Data_Date_vod__c', 'LastModifiedDate'],
                where: 'Contains_Goals_vod__c = _false AND Payer_Plan_Mapped_vod__c = _false AND Status_vod__c = \'Moved_To_Production_vod\' ' +
                'AND Scale_vod__c = _scale AND (Column_Label_vod__c = _dataType OR Second_Column_Label_vod__c = _dataType)',
                sort: ['LastModifiedDate DESC']
            },
            analyticsFilesReport: {
                object: 'Analytics_Files_vod__c',
                fields: ['Column_Label_vod__c','Data_Map_Template_Id_vod__c','Id', 'Market_vod__c', 'Scale_vod__c', 'Second_Column_Label_vod__c','Status_vod__c'],
                where: 'Status_vod__c = \'Moved_To_Production_vod\''
            },
            analyticsProductGroup: {
                object: 'Analytics_Product_Group_vod__c',
                fields: ['Id', 'Market_vod__c'],
                where: 'Id IN '
            },
            analyticsProductGroupForMarket: {
                object: 'Analytics_Product_Group_vod__c',
                fields: ['Id', 'Display_Name_vod__c', 'Display_In_Reports_vod__c', 'Company_Product_Group_vod__c','Parent_Product_Group_vod__c'],
                where: 'Market_vod__c = _marketId'
            },
            analyticsMarket: {
                object: 'Analytics_Markets_vod__c',
                fields: ['Id', 'Name'],
                where: 'Id IN '
            },
            accountSalesSummary: {
                object: 'Account_Sales_Summary_Insights',
                fields: ['Account_Id_vod__c', 'Market_vod__c', 'Market_Id_vod__c', 'Product_Group_vod__c', 'Product_Group_Id_vod__c', 'Sales_Date_vod__c', 'Sales_Data_vod__c'],
                where: 'scale = _scale AND dataType = _dataType AND Market_Id_vod__c = _marketId AND Account_Id_vod__c = _accountId'
            },
            call: {
                object: 'Call2_vod__c',
                fields: ['Id', 'CLM_vod__c', 'Call_Datetime_vod__c'],
                where: 'Account_vod__c = _accountId AND Call_Datetime_vod__c >= _dateTime1 AND Call_Datetime_vod__c < _dateTime2'
            },
            callDetail: {
                object: 'Call2_Detail_vod__c',
                fields: ['Call2_vod__c', 'Product_vod__c'],
                where: 'Call2_vod__c IN '
            },
            callSample: {
                object: 'Call2_Sample_vod__c',
                fields: ['Call2_vod__c', 'Product_vod__c', 'Quantity_vod__c'],
                where: 'Call2_vod__c IN '
            },
            sentEmail: {
                object: 'Sent_Email_vod__c',
                fields: ['Id', 'Opened_vod__c', 'Product_vod__c', 'Email_Sent_Date_vod__c'],
                where: 'Account_vod__c = _accountId AND Email_Sent_Date_vod__c >= _dateTime1 AND Email_Sent_Date_vod__c < _dateTime2'
            }
        })
    ;
})();
